@@ -4,7 +4,7 @@
  *   the Free Software Foundation; either version 2 of the License, or     *
  *   (at your option) any later version.                                   *
  *                                                                         *
- *   copyright (C) 2002-2014                                               *
+ *   copyright (C) 2002-2020                                               *
  *   Umbrello UML Modeller Authors <umbrello-devel@kde.org>                *
  ***************************************************************************/
 
@@ -39,6 +39,7 @@
 #include <QDomDocument>
 #include <QPixmap>
 #include <QTextStream>
+#include <QXmlStreamWriter>
 
 /**
  *  Constructor.
@@ -118,20 +119,21 @@ UMLDragData::~UMLDragData()
  */
 void UMLDragData::setUMLDataClip1(UMLObjectList& objects)
 {
-    QDomDocument domDoc;
-    QDomElement xmiclip = domDoc.createElement(QLatin1String("xmiclip"));
-    domDoc.appendChild(xmiclip);
-    QDomElement objectsTag = domDoc.createElement(QLatin1String("umlobjects"));
-    xmiclip.appendChild(objectsTag);
+    QString xmiClip;
+    QXmlStreamWriter stream(&xmiClip);
+    stream.writeStartElement(QLatin1String("xmiclip"));
+    stream.writeStartElement(QLatin1String("umlobjects"));
 
     UMLObjectListIt object_it(objects);
     UMLObject* obj = 0;
     while (object_it.hasNext()) {
         obj = object_it.next();
-        obj->saveToXMI(domDoc, objectsTag);
+        obj->saveToXMI1(stream);
     }
 
-    setData(QLatin1String("application/x-uml-clip1"), domDoc.toString().toUtf8());
+    stream.writeEndElement();  // umlobjects
+    stream.writeEndElement();  // xmiclip
+    setData(QLatin1String("application/x-uml-clip1"), xmiClip.toUtf8());
 }
 
 /**
@@ -140,27 +142,28 @@ void UMLDragData::setUMLDataClip1(UMLObjectList& objects)
  */
 void UMLDragData::setUMLDataClip2(UMLObjectList& objects, UMLViewList& diagrams)
 {
-    QDomDocument domDoc;
-    QDomElement xmiclip = domDoc.createElement(QLatin1String("xmiclip"));
-    domDoc.appendChild(xmiclip);
-    QDomElement objectsTag = domDoc.createElement(QLatin1String("umlobjects"));
-    xmiclip.appendChild(objectsTag);
+    QString xmiClip;
+    QXmlStreamWriter stream(&xmiClip);
+    stream.writeStartElement(QLatin1String("xmiclip"));
+    stream.writeStartElement(QLatin1String("umlobjects"));
 
     UMLObjectListIt object_it(objects);
     UMLObject* obj = 0;
     while (object_it.hasNext()) {
         obj = object_it.next();
-        obj->saveToXMI(domDoc, objectsTag);
+        obj->saveToXMI1(stream);
     }
 
-    QDomElement viewsTag = domDoc.createElement(QLatin1String("umlviews"));
-    xmiclip.appendChild(viewsTag);
+    stream.writeEndElement();  // umlobjects
+    stream.writeStartElement(QLatin1String("umlviews"));
 
     foreach(UMLView* view, diagrams) {
-        view->umlScene()->saveToXMI(domDoc, viewsTag);
+        view->umlScene()->saveToXMI1(stream);
     }
 
-    setData(QLatin1String("application/x-uml-clip2"), domDoc.toString().toUtf8());
+    stream.writeEndElement();  // umlviews
+    stream.writeEndElement();  // xmiclip
+    setData(QLatin1String("application/x-uml-clip2"), xmiClip.toUtf8());
 }
 
 /**
@@ -169,56 +172,59 @@ void UMLDragData::setUMLDataClip2(UMLObjectList& objects, UMLViewList& diagrams)
  */
 void UMLDragData::setUMLDataClip3(UMLListViewItemList& umlListViewItems)
 {
-    QDomDocument domDoc;
-    QDomElement xmiclip = domDoc.createElement(QLatin1String("xmiclip"));
-    domDoc.appendChild(xmiclip);
-
-    QDomElement itemsTag = domDoc.createElement(QLatin1String("umllistviewitems"));
-    xmiclip.appendChild(itemsTag);
+    QString xmiClip;
+    QXmlStreamWriter stream(&xmiClip);
+    stream.writeStartElement(QLatin1String("xmiclip"));
+    stream.writeStartElement(QLatin1String("umllistviewitems"));
 
     foreach(UMLListViewItem* item, umlListViewItems) {
-        item->saveToXMI(domDoc, itemsTag);
+        item->saveToXMI1(stream);
     }
 
-    setData(QLatin1String("application/x-uml-clip3"), domDoc.toString().toUtf8());
+    stream.writeEndElement();  // umllistviewitems
+    stream.writeEndElement();  // xmiclip
+    setData(QLatin1String("application/x-uml-clip3"), xmiClip.toUtf8());
 }
 
 /**
  * For use when the user selects UML Objects from a
- * Diagram. The Selected widegets and the relationships
+ * Diagram. The Selected widgets and the relationships
  * between only selected widgets will be copied and also
  * its respective ListView Items
  */
-void UMLDragData::setUMLDataClip4(UMLObjectList& objects, UMLWidgetList& widgets, AssociationWidgetList& associations,
+void UMLDragData::setUMLDataClip4(UMLObjectList& objects,
+                                  UMLWidgetList& widgets,
+                                  AssociationWidgetList& associations,
                                   QPixmap& pngImage, UMLScene *scene)
 {
-    QDomDocument domDoc;
-    QDomElement xmiclip = domDoc.createElement(QLatin1String("xmiclip"));
-    xmiclip.setAttribute(QLatin1String("diagramtype"), scene->type());
-    xmiclip.setAttribute(QLatin1String("diagramid"), Uml::ID::toString(scene->ID()));
-    domDoc.appendChild(xmiclip);
-    QDomElement objectsTag = domDoc.createElement(QLatin1String("umlobjects"));
-    xmiclip.appendChild(objectsTag);
+    QString xmiClip;
+    QXmlStreamWriter stream(&xmiClip);
+    stream.writeStartElement(QLatin1String("xmiclip"));
+    stream.writeAttribute(QLatin1String("diagramtype"), QString::number(scene->type()));
+    stream.writeAttribute(QLatin1String("diagramid"), Uml::ID::toString(scene->ID()));
+    stream.writeStartElement(QLatin1String("umlobjects"));
 
     foreach (UMLObject* obj, objects) {
-        obj->saveToXMI(domDoc, objectsTag);
+        obj->saveToXMI1(stream);
     }
 
-    QDomElement widgetsTag = domDoc.createElement(QLatin1String("widgets"));
-    xmiclip.appendChild(widgetsTag);
+    stream.writeEndElement();  // umlobjects
+    stream.writeStartElement(QLatin1String("widgets"));
 
     foreach (UMLWidget* widget, widgets) {
-        widget->saveToXMI(domDoc, widgetsTag);
+        widget->saveToXMI1(stream);
     }
 
-    QDomElement associationWidgetsTag = domDoc.createElement(QLatin1String("associations"));
-    xmiclip.appendChild(associationWidgetsTag);
+    stream.writeEndElement();  // widgets
+    stream.writeStartElement(QLatin1String("associations"));
 
     foreach (AssociationWidget* association, associations) {
-        association->saveToXMI(domDoc, associationWidgetsTag);
+        association->saveToXMI1(stream);
     }
 
-    setData(QLatin1String("application/x-uml-clip4"), domDoc.toString().toUtf8());
+    stream.writeEndElement();  // associations
+    stream.writeEndElement();  // xmiclip
+    setData(QLatin1String("application/x-uml-clip4"), xmiClip.toUtf8());
 
     QImage img = pngImage.toImage();
     int l_size = img.byteCount();
@@ -231,22 +237,23 @@ void UMLDragData::setUMLDataClip4(UMLObjectList& objects, UMLWidgetList& widgets
 }
 
 /**
- * For use when the user selects only Attirbutes and/or
+ * For use when the user selects only Attributes and/or
  * Operation from the ListView
  */
 void UMLDragData::setUMLDataClip5(UMLObjectList& objects)
 {
-    QDomDocument domDoc;
-    QDomElement xmiclip = domDoc.createElement(QLatin1String("xmiclip"));
-    domDoc.appendChild(xmiclip);
-    QDomElement objectsTag = domDoc.createElement(QLatin1String("umlobjects"));
-    xmiclip.appendChild(objectsTag);
+    QString xmiClip;
+    QXmlStreamWriter stream(&xmiClip);
+    stream.writeStartElement(QLatin1String("xmiclip"));
+    stream.writeStartElement(QLatin1String("umlobjects"));
 
     foreach (UMLObject* obj, objects) {
-        obj->saveToXMI(domDoc, objectsTag);
+        obj->saveToXMI1(stream);
     }
 
-    setData(QLatin1String("application/x-uml-clip5"), domDoc.toString().toUtf8());
+    stream.writeEndElement();  // umlobjects
+    stream.writeEndElement();  // xmiclip
+    setData(QLatin1String("application/x-uml-clip5"), xmiClip.toUtf8());
 }
 
 /**
@@ -326,7 +333,7 @@ bool UMLDragData::decodeClip2(const QMimeData* mimeData, UMLObjectList& objects,
 
     // Load UMLObjects
     QDomNode objectsNode = xmiClipNode.firstChild();
-    if (NoteWidget::s_pCurrentNote == NULL) {
+    if (NoteWidget::s_pCurrentNote == 0) {
         if (!UMLDragData::decodeObjects(objectsNode, objects, true)) {
             return false;
         }
@@ -461,7 +468,7 @@ bool UMLDragData::decodeClip3(const QMimeData* mimeData, UMLListViewItemList& um
         UMLListViewItem::ListViewType t = (UMLListViewItem::ListViewType)(type.toInt());
         UMLListViewItem* parent = parentListView->determineParentItem(t);
         UMLListViewItem* itemData = new UMLListViewItem(parent);
-        if (itemData->loadFromXMI(listItemElement))
+        if (itemData->loadFromXMI1(listItemElement))
             umlListViewItems.append(itemData);
         else
             delete itemData;
@@ -526,7 +533,9 @@ bool UMLDragData::decodeClip4(const QMimeData* mimeData, UMLObjectList& objects,
     QString sourceDiagramID = root.attribute(QLatin1String("diagramid"), QLatin1String(""));
     UMLView *sourceView = doc->findView(Uml::ID::fromString(sourceDiagramID));
 
-    bool pasteToDiagramCopiedFrom = sourceView && sourceView->umlScene()->ID() == scene->ID();
+    bool fromSameDiagram = sourceView && sourceView->umlScene()->ID() == scene->ID();
+    bool fromAnotherInstance = !sourceView;
+    bool fromDifferentDiagramType = dType != scene->type();
 
     // Load widgets
     QDomNode widgetsNode = objectsNode.nextSibling();
@@ -541,7 +550,7 @@ bool UMLDragData::decodeClip4(const QMimeData* mimeData, UMLObjectList& objects,
 
         UMLWidget* widget = scene->loadWidgetFromXMI(widgetElement);
         if (widget) {
-            if (pasteToDiagramCopiedFrom && widget->baseType() == WidgetBase::wt_Object) {
+            if (fromSameDiagram && widget->isObjectWidget()) {
                 delete widget;
                 widgetNode = widgetNode.nextSibling();
                 widgetElement = widgetNode.toElement();
@@ -549,7 +558,7 @@ bool UMLDragData::decodeClip4(const QMimeData* mimeData, UMLObjectList& objects,
             }
 
             // check if widget is pastable
-            if (!sourceView || sourceView->umlScene()->type() != scene->type()) {
+            if (fromAnotherInstance || fromDifferentDiagramType) {
                 UMLObject *object = widget->umlObject();
                 if (object) {
                     if (!Model_Utils::typeIsAllowedInDiagram(object, scene)) {
@@ -565,14 +574,28 @@ bool UMLDragData::decodeClip4(const QMimeData* mimeData, UMLObjectList& objects,
                     widgetElement = widgetNode.toElement();
                     continue;
                 }
+            } else if (Model_Utils::isCloneable(widget->baseType())) {
+                if (widget->umlObject()) {
+                    UMLObject *clone = widget->umlObject()->clone();
+                    widget->setUMLObject(clone);
+                    // we do not want to recreate an additional widget,
+                    // which would be the case if calling scene->addUMLObject()
+                    UMLApp::app()->document()->addUMLObject(clone);
+                    UMLApp::app()->listView()->slotObjectCreated(clone);
+                    if (Model_Utils::hasAssociations(clone->baseType()))
+                    {
+                        scene->createAutoAssociations(widget);
+                        scene->createAutoAttributeAssociations2(widget);
+                    }
+                }
             }
 
             // Generate a new unique 'local ID' so a second widget for the same
             // UMLObject can be distinguished from the first widget
             widget->setLocalID(doc->assignNewID(widget->localID()));
 
-            if (widget->baseType() == WidgetBase::wt_Message) {
-                MessageWidget *w = static_cast<MessageWidget*>(widget);
+            if (widget->isMessageWidget()) {
+                MessageWidget *w = widget->asMessageWidget();
                 if (w && w->floatingTextWidget()) {
                     w->floatingTextWidget()->setLocalID(doc->assignNewID(w->floatingTextWidget()->localID()));
                     w->floatingTextWidget()->setID(doc->assignNewID(w->floatingTextWidget()->id()));
@@ -596,9 +619,9 @@ bool UMLDragData::decodeClip4(const QMimeData* mimeData, UMLObjectList& objects,
 
     // Make sure all object widgets are loaded before adding messages or
     // preconditions
-    if (!pasteToDiagramCopiedFrom) {
+    if (!fromSameDiagram) {
         foreach (UMLWidget* widget, widgets) {
-            if (widget->baseType() == WidgetBase::wt_Object) {
+            if (widget->isObjectWidget()) {
                 executeCreateWidgetCommand(widget);
             }
         }
@@ -606,17 +629,17 @@ bool UMLDragData::decodeClip4(const QMimeData* mimeData, UMLObjectList& objects,
 
     // Now add all remaining widgets
     foreach (UMLWidget* widget, widgets) {
-        if (!pasteToDiagramCopiedFrom && widget->baseType() == WidgetBase::wt_Message) {
-            MessageWidget* message = dynamic_cast<MessageWidget*>(widget);
+        if (!fromSameDiagram && widget->isMessageWidget()) {
+            MessageWidget* message = widget->asMessageWidget();
             message->resolveObjectWidget(log);
         }
 
-        if (widget->baseType() == WidgetBase::wt_Precondition) {
-            PreconditionWidget* precondition = dynamic_cast<PreconditionWidget*>(widget);
+        if (widget->isPreconditionWidget()) {
+            PreconditionWidget* precondition = widget->asPreconditionWidget();
             precondition->resolveObjectWidget(log);
         }
 
-        if (widget->baseType() != WidgetBase::wt_Object) {
+        if (!widget->isObjectWidget()) {
             executeCreateWidgetCommand(widget);
         }
     }
@@ -627,7 +650,7 @@ bool UMLDragData::decodeClip4(const QMimeData* mimeData, UMLObjectList& objects,
     QDomElement associationWidgetElement = associationWidgetNode.toElement();
     while (!associationWidgetElement.isNull()) {
         AssociationWidget* associationWidget = AssociationWidget::create(view->umlScene());
-        if (associationWidget->loadFromXMI(associationWidgetElement, widgets, 0))
+        if (associationWidget->loadFromXMI1(associationWidgetElement, widgets, 0))
             associations.append(associationWidget);
         else {
             delete associationWidget;
@@ -687,7 +710,7 @@ bool UMLDragData::decodeClip5(const QMimeData* mimeData, UMLObjectList& objects,
             uWarning() << "Given wrong type of umlobject to create:" << type;
             return false;
         }
-        if(!pObject->loadFromXMI(element)) {
+        if(!pObject->loadFromXMI1(element)) {
             uWarning() << "Failed to load object from XMI.";
             return false;
         }
@@ -775,7 +798,7 @@ bool UMLDragData::decodeObjects(QDomNode& objectsNode, UMLObjectList& objects, b
                 newParentId = oldParentId;
             }
 
-            newParent = static_cast<UMLPackage*>(doc->findObjectById(newParentId));
+            newParent = doc->findObjectById(newParentId)->asUMLPackage();
         }
 
         if (newParent == 0) {
@@ -786,9 +809,9 @@ bool UMLDragData::decodeObjects(QDomNode& objectsNode, UMLObjectList& objects, b
 
         pObject->setUMLPackage(newParent);
 
-        // Note: element should not be used after calling loadFromXMI() because
+        // Note: element should not be used after calling loadFromXMI1() because
         // it can point to an arbitrary child node
-        if(!pObject->loadFromXMI(element)) {
+        if(!pObject->loadFromXMI1(element)) {
             uWarning() << "Failed to load object of type " << type << " from XMI";
             delete pObject;
             return false;
@@ -839,16 +862,16 @@ bool UMLDragData::decodeViews(QDomNode& umlviewsNode, UMLViewList& diagrams)
         QString type = diagramElement.attribute(QLatin1String("type"), QLatin1String("0"));
         Uml::DiagramType::Enum dt = Uml::DiagramType::fromInt(type.toInt());
         UMLListViewItem *parent = listView->findFolderForDiagram(dt);
-        if (parent == NULL)
+        if (parent == 0)
             return false;
         UMLObject *po = parent->umlObject();
-        if (po == NULL || po->baseType() != UMLObject::ot_Folder) {
+        if (po == 0 || po->baseType() != UMLObject::ot_Folder) {
             uError() << "Bad parent for view.";
             return false;
         }
-        UMLFolder *f = static_cast<UMLFolder*>(po);
+        UMLFolder *f = po->asUMLFolder();
         UMLView* view = new UMLView(f);
-        view->umlScene()->loadFromXMI(diagramElement);
+        view->umlScene()->loadFromXMI1(diagramElement);
         diagrams.append(view);
         diagramNode = diagramNode.nextSibling();
         diagramElement = diagramNode.toElement();
@@ -878,6 +901,9 @@ int UMLDragData::getCodingType(const QMimeData* mimeData)
     if (mimeData->hasFormat(QLatin1String("application/x-uml-clip5"))) {
         result = 5;
     }
+    if (mimeData->hasFormat(QLatin1String("text/plain"))) {
+        result = 6;
+    }
     return result;
 }
 
