@@ -4,7 +4,7 @@
  *   the Free Software Foundation; either version 2 of the License, or     *
  *   (at your option) any later version.                                   *
  *                                                                         *
- *   copyright (C) 2002-2014                                               *
+ *   copyright (C) 2002-2020                                               *
  *   Umbrello UML Modeller Authors <umbrello-devel@kde.org>                *
  ***************************************************************************/
 
@@ -12,6 +12,7 @@
 
 // app includes
 #include "associationwidget.h"
+#include "debug_utils.h"
 #include "model_utils.h"
 #include "uml.h"
 #include "umlscene.h"
@@ -19,6 +20,7 @@
 
 // kde includes
 #include <KLocalizedString>
+#include <QXmlStreamWriter>
 
 namespace Uml
 {
@@ -33,9 +35,19 @@ namespace Uml
 
         addWidgetToScene(widget);
 
-        QDomDocument doc;
-        m_element = doc.createElement(QLatin1String("widget"));
-        widget->saveToXMI1(doc, m_element);
+        QString xmi;
+        QXmlStreamWriter stream(&xmi);
+        stream.writeStartElement(QLatin1String("widget"));
+        widget->saveToXMI1(stream);
+        stream.writeEndElement();  // widget
+        QString error;
+        int line;
+        QDomDocument domDoc;
+        if (domDoc.setContent(xmi, &error, &line)) {
+            m_element = domDoc.firstChild().firstChild().toElement();
+        } else {
+            uWarning() << "Cannot set content:" << error << " line:" << line;
+        }
     }
 
     /**
@@ -48,9 +60,19 @@ namespace Uml
 
         addWidgetToScene(widget);
 
-        QDomDocument doc;
-        m_element = doc.createElement(QLatin1String("widget"));
-        widget->saveToXMI1(doc, m_element);
+        QString xmi;
+        QXmlStreamWriter stream(&xmi);
+        stream.writeStartElement(QLatin1String("widget"));
+        widget->saveToXMI1(stream);
+        stream.writeEndElement();  // widget
+        QString error;
+        int line;
+        QDomDocument domDoc;
+        if (domDoc.setContent(xmi, &error, &line)) {
+            m_element = domDoc.firstChild().firstChild().toElement();
+        } else {
+            uWarning() << "Cannot set content:" << error << " line:" << line;
+        }
     }
 
     /**
@@ -78,18 +100,23 @@ namespace Uml
             }
         } else {
             AssociationWidget* widget = scene()->findAssocWidget(m_widgetId);
-            if (widget == nullptr) {
-                // If the widget is not found, the add command was undone. Load the
-                // widget back from the saved XMI state.
-                QDomElement widgetElement = m_element.firstChild().toElement();
-                widget = AssociationWidget::create(scene());
-                if (widget->loadFromXMI1(widgetElement)) {
-                    addWidgetToScene(widget);
-                    m_assocWidget = widget;
-                    m_widgetId = widget->id();
-                } else
-                    delete widget;
+            if (widget)
+                return;
+            if (m_assocWidget) {
+                scene()->addAssociation(m_assocWidget, false);
+                return;
             }
+
+            // If the widget is not found, the add command was undone. Load the
+            // widget back from the saved XMI state.
+            QDomElement widgetElement = m_element.firstChild().toElement();
+            widget = AssociationWidget::create(scene());
+            if (widget->loadFromXMI1(widgetElement)) {
+                addWidgetToScene(widget);
+                m_assocWidget = widget;
+                m_widgetId = widget->id();
+            } else
+                delete widget;
         }
     }
 
