@@ -5,7 +5,7 @@
  *   (at your option) any later version.                                   *
  *                                                                         *
  *   copyright (C) 2003      Brian Thomas <thomas@mail630.gsfc.nasa.gov>   *
- *   copyright (C) 2004-2014                                               *
+ *   copyright (C) 2004-2020                                               *
  *   Umbrello UML Modeller Authors <umbrello-devel@kde.org>                *
  ***************************************************************************/
 
@@ -28,18 +28,18 @@ CodeComment::~CodeComment()
 /**
  * Save the XMI representation of this object.
  */
-void CodeComment::saveToXMI (QDomDocument & doc, QDomElement & root)
+void CodeComment::saveToXMI1(QXmlStreamWriter& writer)
 {
-    QDomElement blockElement = doc.createElement(QLatin1String("codecomment"));
-    setAttributesOnNode(doc, blockElement); // as we added no additional fields to this class we may
+    writer.writeStartElement(QLatin1String("codecomment"));
+    setAttributesOnNode(writer); // as we added no additional fields to this class we may
     // just use parent TextBlock method
-    root.appendChild(blockElement);
+    writer.writeEndElement();
 }
 
 /**
  * Load params from the appropriate XMI element node.
  */
-void CodeComment::loadFromXMI(QDomElement & root)
+void CodeComment::loadFromXMI1(QDomElement & root)
 {
     setAttributesFromNode(root);
 }
