@@ -4,7 +4,7 @@
  *   the Free Software Foundation; either version 2 of the License, or     *
  *   (at your option) any later version.                                   *
  *                                                                         *
- *   copyright (C) 2018                                                    *
+ *   copyright (C) 2018-2020                                               *
  *   Umbrello UML Modeller Authors <umbrello-devel@kde.org>                *
  ***************************************************************************/
 
@@ -26,6 +26,7 @@
 #include "pinportbase.h"
 #include "statewidget.h"
 #include "uml.h"
+#include "umldoc.h"
 #include "umllistview.h"
 #include "umlscene.h"
 
@@ -104,7 +105,7 @@ void WidgetBasePopupMenu::makeClassifierShowPopup(ClassifierWidget *c)
     insert(mt_Show_Packages, show, i18n("Package"), CHECKABLE);
     setActionChecked(mt_Show_Packages, c->visualProperty(ClassifierWidget::ShowPackage));
     insert(mt_Show_Stereotypes, show, i18n("Stereotype"), CHECKABLE);
-    setActionChecked(mt_Show_Stereotypes, c->visualProperty(ClassifierWidget::ShowStereotype));
+    setActionChecked(mt_Show_Stereotypes, c->showStereotype() != Uml::ShowStereoType::None);
     addMenu(show);
 }
 
@@ -190,6 +191,13 @@ void WidgetBasePopupMenu::insertSingleSelectionMenu(WidgetBase* widget)
             insertSubMenuNew(type);
             makeClassifierShowPopup(c);
             insertSubMenuColor(c->useFillColor());
+            addSeparator();
+            if (c->linkedDiagram())
+                insert(mt_GoToStateDiagram);
+            if (UMLApp::app()->document()->views(Uml::DiagramType::State).size() > 0)
+                insert(mt_SelectStateDiagram);
+            if (c->linkedDiagram())
+                insert(mt_RemoveStateDiagram);
             insertStdItems(true, type);
             insert(mt_Rename);
             insert(mt_Change_Font);
@@ -299,6 +307,11 @@ void WidgetBasePopupMenu::insertSingleSelectionMenu(WidgetBase* widget)
         insertStdItems(true, type);
         insert(mt_Rename_Object);
         insert(mt_Rename, i18n("Rename Class..."));
+        if (widget->asObjectWidget() &&
+                widget->umlScene() &&
+                widget->umlScene()->isSequenceDiagram()) {
+            insert(widget->asObjectWidget()->showDestruction() ? mt_Hide_Destruction_Box : mt_Show_Destruction_Box);
+        }
         insert(mt_Change_Font);
         insert(mt_Properties);
         break;
@@ -341,6 +354,7 @@ void WidgetBasePopupMenu::insertSingleSelectionMenu(WidgetBase* widget)
             switch (pState->stateType()) {
             case StateWidget::Combined:
                 insert(mt_EditCombinedState);
+                addSeparator();
                 break;
             default:
                 break;
@@ -348,7 +362,7 @@ void WidgetBasePopupMenu::insertSingleSelectionMenu(WidgetBase* widget)
 
             insertSubMenuNew(type);
             insertSubMenuColor(widget->useFillColor());
-            insertStdItems(false, type);
+            insertStdItems(true, type);
             switch (pState->stateType()) {
             case StateWidget::Combined:
                 insert(mt_Change_Font);
@@ -440,6 +454,8 @@ void WidgetBasePopupMenu::insertSingleSelectionMenu(WidgetBase* widget)
         }
         insert(mt_Delete);
         insert(mt_Change_Font);
+        if (type == WidgetBase::wt_Precondition)
+            insert(mt_Properties);
         break;
 
     case WidgetBase::wt_CombinedFragment:
@@ -460,6 +476,7 @@ void WidgetBasePopupMenu::insertSingleSelectionMenu(WidgetBase* widget)
         insert(mt_Rename, i18n("Change Text..."));
         insert(mt_Delete);
         insert(mt_Change_Font);
+        insert(mt_Properties);
         break;
 
     case WidgetBase::wt_Text:
@@ -661,6 +678,7 @@ void WidgetBasePopupMenu::insertSubMenuNew(WidgetBase::WidgetType type, KMenu *m
             insert(mt_Interface, menu);
             insert(mt_Datatype, menu);
             insert(mt_Enum, menu);
+            insert(mt_State_Diagram, menu);
             break;
         case WidgetBase::wt_Interface:
             insert(mt_Operation, menu);
