@@ -4,7 +4,7 @@
  *   the Free Software Foundation; either version 2 of the License, or     *
  *   (at your option) any later version.                                   *
  *                                                                         *
- *   copyright (C) 2004-2020                                               *
+ *   copyright (C) 2004-2021                                               *
  *   Umbrello UML Modeller Authors <umbrello-devel@kde.org>                *
  ***************************************************************************/
 
@@ -21,6 +21,7 @@
 #include "folder.h"
 #include "classifier.h"
 #include "enum.h"
+#include "instance.h"
 #include "entity.h"
 #include "template.h"
 #include "operation.h"
@@ -101,7 +102,6 @@ UMLObject* findObjectInList(Uml::ID::Type id, const UMLObjectList& inList)
         case UMLObject::ot_Interface:
         case UMLObject::ot_Class:
         case UMLObject::ot_Enum:
-        case UMLObject::ot_Instance:
             o = obj->asUMLClassifier()->findChildObjectById(id);
             if (o == nullptr &&
                     (t == UMLObject::ot_Interface || t == UMLObject::ot_Class))
@@ -109,6 +109,11 @@ UMLObject* findObjectInList(Uml::ID::Type id, const UMLObjectList& inList)
             if (o)
                 return o;
             break;
+        case UMLObject::ot_Instance:
+            o = obj->asUMLInstance()->findChildObjectById(id);
+            if (o)
+                return o;
+            break;
         case UMLObject::ot_Entity:
             o = obj->asUMLEntity()->findChildObjectById(id);
             if (o)
@@ -558,7 +563,7 @@ QString uniqObjectName(UMLObject::ObjectType type, UMLPackage *parentPkg, QStrin
         case UMLObject::ot_UseCase:             currentName = i18n("new_use case");               break;
         default:
             currentName = i18n("new_object");
-            uWarning() << "unknown object type";
+            uWarning() << "model_utils::uniqObjectName unknown object type" << UMLObject::toString(type);
         }
     }
     UMLDoc *doc = UMLApp::app()->document();
@@ -606,7 +611,7 @@ QString newTitle(UMLObject::ObjectType type)
     case UMLObject::ot_UniqueConstraint:    return i18n("New unique constraint");
     case UMLObject::ot_UseCase:             return i18n("New use case");
     default:
-        uWarning() << "unknown object type" << UMLObject::toString(type);
+        uWarning() << "model_utils::newTitle unknown object type" << UMLObject::toString(type);
         return i18n("New UML object");
     }
     return QString();
@@ -649,7 +654,7 @@ QString newText(UMLObject::ObjectType type)
     case UMLObject::ot_UniqueConstraint:    return i18n("Enter the name of the new unique constraint:");
     case UMLObject::ot_UseCase:             return i18n("Enter the name of the new use case:");
     default:
-        uWarning() << "unknown object type" << UMLObject::toString(type);
+        uWarning() << "model_utils::newText unknown object type" << UMLObject::toString(type);
         return i18n("Enter the name of the new UML object");
     }
     return QString();
@@ -692,7 +697,7 @@ QString renameTitle(UMLObject::ObjectType type)
     case UMLObject::ot_UniqueConstraint:    return i18n("Rename unique constraint");
     case UMLObject::ot_UseCase:             return i18n("Rename use case");
     default:
-        uWarning() << "unknown object type" << UMLObject::toString(type);
+        uWarning() << "model_utils::renameTitle unknown object type" << UMLObject::toString(type);
         return i18n("Rename UML object");
     }
     return QString();
@@ -735,7 +740,7 @@ QString renameText(UMLObject::ObjectType type)
     case UMLObject::ot_UniqueConstraint:    return i18n("Enter the new name of the unique constraint:");
     case UMLObject::ot_UseCase:             return i18n("Enter the new name of the use case:");
     default:
-        uWarning() << "unknown object type" << UMLObject::toString(type);
+        uWarning() << "model_utils::renameText unknown object type" << UMLObject::toString(type);
         return i18n("Enter the new name of the UML object");
     }
     return QString();
@@ -787,7 +792,7 @@ bool isCommonDataType(QString type)
 {
     CodeGenerator *gen = UMLApp::app()->generator();
     if (gen == 0)
-        return false;
+        return true;   // gen == NULL means we use UMLPrimitiveTypes
     const bool caseSensitive = UMLApp::app()->activeLanguageIsCaseSensitive();
     const QStringList dataTypes = gen->defaultDatatypes();
     QStringList::ConstIterator end(dataTypes.end());
@@ -814,8 +819,9 @@ bool isClassifierListitem(UMLObject::ObjectType type)
         type == UMLObject::ot_EnumLiteral ||
         type == UMLObject::ot_UniqueConstraint ||
         type == UMLObject::ot_ForeignKeyConstraint  ||
-        type == UMLObject::ot_CheckConstraint ||
-        type == UMLObject::ot_InstanceAttribute ) {
+        type == UMLObject::ot_CheckConstraint) {
+        // UMLObject::ot_InstanceAttribute needs to be handled separately
+        // because UMLInstanceAttribute is not a UMLClassifierListItem.
         return true;
     } else {
         return false;
@@ -1197,6 +1203,7 @@ bool typeIsCanvasWidget(UMLListViewItem::ListViewType type)
         case UMLListViewItem::lvt_Interface:
         case UMLListViewItem::lvt_Datatype:
         case UMLListViewItem::lvt_Enum:
+        case UMLListViewItem::lvt_Instance:
         case UMLListViewItem::lvt_Entity:
         case UMLListViewItem::lvt_Category:
             return true;
@@ -1246,7 +1253,6 @@ bool typeIsContainer(UMLListViewItem::ListViewType type)
 bool typeIsClassifierList(UMLListViewItem::ListViewType type)
 {
     if (type == UMLListViewItem::lvt_Attribute ||
-        type == UMLListViewItem::lvt_Instance ||
         type == UMLListViewItem::lvt_Operation ||
         type == UMLListViewItem::lvt_Template ||
         type == UMLListViewItem::lvt_EntityAttribute ||
@@ -1254,8 +1260,9 @@ bool typeIsClassifierList(UMLListViewItem::ListViewType type)
         type == UMLListViewItem::lvt_ForeignKeyConstraint ||
         type == UMLListViewItem::lvt_PrimaryKeyConstraint ||
         type == UMLListViewItem::lvt_CheckConstraint  ||
-        type == UMLListViewItem::lvt_EnumLiteral ||
-        type == UMLListViewItem::lvt_InstanteAttribute) {
+        type == UMLListViewItem::lvt_EnumLiteral) {
+        //  UMLListViewItem::lvt_InstanceAttribute must be handled separately
+        //  because UMLInstanceAttribute is not a UMLClassifierListItem.
         return true;
     } else {
         return false;
@@ -1304,7 +1311,7 @@ bool typeIsProperties(UMLListViewItem::ListViewType type)
  * as child of type parentType
  */
 bool typeIsAllowedInType(UMLListViewItem::ListViewType childType,
-    UMLListViewItem::ListViewType parentType)
+                         UMLListViewItem::ListViewType parentType)
 {
     switch (childType) {
     case UMLListViewItem::lvt_Class:
@@ -1317,9 +1324,11 @@ bool typeIsAllowedInType(UMLListViewItem::ListViewType childType,
                parentType == UMLListViewItem::lvt_Package ||
                parentType == UMLListViewItem::lvt_Logical_Folder;
     case UMLListViewItem::lvt_Attribute:
+        return parentType == UMLListViewItem::lvt_Class;
     case UMLListViewItem::lvt_EntityAttribute:
-    case UMLListViewItem::lvt_InstanteAttribute:
         return parentType == UMLListViewItem::lvt_Entity;
+    case UMLListViewItem::lvt_InstanceAttribute:
+        return parentType == UMLListViewItem::lvt_Instance;
     case UMLListViewItem::lvt_Operation:
         return parentType == UMLListViewItem::lvt_Class ||
                parentType == UMLListViewItem::lvt_Interface;
@@ -1338,7 +1347,8 @@ bool typeIsAllowedInType(UMLListViewItem::ListViewType childType,
         return parentType == UMLListViewItem::lvt_Logical_Folder ||
                parentType == UMLListViewItem::lvt_Logical_View;
     case UMLListViewItem::lvt_Logical_Folder:
-        return parentType == UMLListViewItem::lvt_Logical_Folder ||
+        return parentType == UMLListViewItem::lvt_Package ||
+               parentType == UMLListViewItem::lvt_Logical_Folder ||
                parentType == UMLListViewItem::lvt_Logical_View;
     case UMLListViewItem::lvt_UseCase_Folder:
         return parentType == UMLListViewItem::lvt_UseCase_Folder ||
@@ -1401,7 +1411,7 @@ bool typeIsDiagram(UMLListViewItem::ListViewType type)
             type == UMLListViewItem::lvt_Component_Diagram ||
             type == UMLListViewItem::lvt_Deployment_Diagram ||
             type == UMLListViewItem::lvt_EntityRelationship_Diagram ||
-            type == UMLListViewItem::lvt_Object_Diagram ){
+            type == UMLListViewItem::lvt_Object_Diagram) {
         return true;
     } else {
         return false;
@@ -1592,9 +1602,9 @@ UMLListViewItem::ListViewType convert_OT_LVT(UMLObject *o)
     case UMLObject::ot_Folder:
         {
             UMLDoc *umldoc = UMLApp::app()->document();
-            UMLFolder *f = o->asUMLFolder();
+            UMLPackage *p = o->asUMLPackage();
             do {
-                const Uml::ModelType::Enum mt = umldoc->rootFolderType(f);
+                const Uml::ModelType::Enum mt = umldoc->rootFolderType(p);
                 if (mt != Uml::ModelType::N_MODELTYPES) {
                     switch (mt) {
                         case Uml::ModelType::Logical:
@@ -1617,7 +1627,7 @@ UMLListViewItem::ListViewType convert_OT_LVT(UMLObject *o)
                     }
                     return type;
                 }
-            } while ((f = f->umlPackage()->asUMLFolder()) != 0);
+            } while ((p = p->umlPackage()) != 0);
             uError() << "convert_OT_LVT(" << o->name()
                 << "): internal error - object is not properly nested in folder";
         }
@@ -1707,7 +1717,7 @@ UMLListViewItem::ListViewType convert_OT_LVT(UMLObject *o)
         break;
 
     case UMLObject::ot_InstanceAttribute:
-        type = UMLListViewItem::lvt_InstanteAttribute;
+        type = UMLListViewItem::lvt_InstanceAttribute;
         break;
 
     default:
@@ -1820,7 +1830,7 @@ UMLObject::ObjectType convert_LVT_OT(UMLListViewItem::ListViewType lvt)
         ot = UMLObject::ot_Instance;
         break;
 
-    case UMLListViewItem::lvt_InstanteAttribute:
+    case UMLListViewItem::lvt_InstanceAttribute:
         ot = UMLObject::ot_InstanceAttribute;
         break;
 
@@ -2001,7 +2011,7 @@ Icon_Utils::IconType convert_LVT_IT(UMLListViewItem::ListViewType lvt, UMLObject
         case UMLListViewItem::lvt_Instance:
             icon = Icon_Utils::it_Instance;
         break;
-        case UMLListViewItem::lvt_InstanteAttribute:
+        case UMLListViewItem::lvt_InstanceAttribute:
             icon = Icon_Utils::it_Private_Attribute;
         break;
         default:
@@ -2186,18 +2196,25 @@ bool typeIsAllowedInDiagram(UMLObject* o, UMLScene *scene)
              ot != UMLObject::ot_Package &&
              ot != UMLObject::ot_Interface &&
              ot != UMLObject::ot_Enum &&
-             ot != UMLObject::ot_Datatype)) {
+             ot != UMLObject::ot_Datatype &&
+             ot != UMLObject::ot_Instance)) {
             bAccept = false;
         }
         break;
     case Uml::DiagramType::Object:
-        if( scene->widgetOnDiagram(id) || (ot != UMLObject::ot_Instance))
-                bAccept = false;
+        if (scene->widgetOnDiagram(id) || ot != UMLObject::ot_Instance)
+            bAccept = false;
         break;
     case Uml::DiagramType::Sequence:
+        if (ot != UMLObject::ot_Class &&
+            ot != UMLObject::ot_Interface &&
+            ot != UMLObject::ot_Actor)
+            bAccept = false;
+        break;
     case Uml::DiagramType::Collaboration:
         if (ot != UMLObject::ot_Class &&
             ot != UMLObject::ot_Interface &&
+            ot != UMLObject::ot_Instance &&
             ot != UMLObject::ot_Actor)
             bAccept = false;
         break;
