@@ -4,7 +4,7 @@
  *   the Free Software Foundation; either version 2 of the License, or     *
  *   (at your option) any later version.                                   *
  *                                                                         *
- *   copyright (C) 2004-2020                                               *
+ *   copyright (C) 2004-2021                                               *
  *   Umbrello UML Modeller Authors <umbrello-devel@kde.org>                *
  ***************************************************************************/
 
@@ -167,7 +167,7 @@ void ToolBarStateAssociation::mouseReleaseEmpty()
  * Sets the first widget in the association using the current widget.
  * If the widget can't be associated using the current type of association,
  * an error is shown and the widget isn't set.
- * Otherwise, the temporal visual association is created and the mouse
+ * Otherwise, the temporary visual association is created and the mouse
  * tracking is enabled, so move events will be delivered.
  */
 void ToolBarStateAssociation::setFirstWidget()
@@ -263,6 +263,11 @@ void ToolBarStateAssociation::setSecondWidget()
             } */
         }
         AssociationWidget *temp = AssociationWidget::create(m_pUMLScene, widgetA, type, widgetB);
+        if (widgetA->baseType() == UMLWidget::wt_Port) {
+            QPointF lineStart = widgetA->getPos();
+            uDebug() << "ToolBarStateAssociation::setSecondWidget : lineStart = " << lineStart;
+            temp->associationLine()->setPoint(0, lineStart);
+        }
         FloatingTextWidget *wt = temp->textWidgetByRole(Uml::TextRole::Coll_Message);
         if (wt)
             wt->showOperationDialog();
@@ -302,8 +307,8 @@ Uml::AssociationType::Enum ToolBarStateAssociation::getAssociationType()
         case WorkToolBar::tbb_Seq_Combined_Fragment:
         case WorkToolBar::tbb_Seq_Precondition:
         case WorkToolBar::tbb_Seq_Message_Asynchronous: at = Uml::AssociationType::Seq_Message;       break;
-        case WorkToolBar::tbb_Coll_Message_Synchronous: at = Uml::AssociationType::Coll_Message_Synchronous;      break;
-        case WorkToolBar::tbb_Coll_Message_Asynchronous: at = Uml::AssociationType::Coll_Message_Asynchronous;      break;
+        case WorkToolBar::tbb_Coll_Mesg_Sync: at = Uml::AssociationType::Coll_Mesg_Sync;      break;
+        case WorkToolBar::tbb_Coll_Mesg_Async: at = Uml::AssociationType::Coll_Mesg_Async;      break;
         case WorkToolBar::tbb_State_Transition:         at = Uml::AssociationType::State;             break;
         case WorkToolBar::tbb_Activity_Transition:      at = Uml::AssociationType::Activity;          break;
         case WorkToolBar::tbb_Exception:                at = Uml::AssociationType::Exception;         break;
@@ -346,7 +351,7 @@ bool ToolBarStateAssociation::addAssociationInViewAndDoc(AssociationWidget* asso
 }
 
 /**
- * Cleans the first widget and the temporal association line, if any.
+ * Cleans the first widget and the temporary association line, if any.
  * Both are set to null, and the association line is also deleted.
  */
 void ToolBarStateAssociation::cleanAssociation()
