@@ -62,7 +62,7 @@ public:
     void setReferencedEntity(UMLEntity* ent);
     UMLEntity* getReferencedEntity() const;
 
-    void saveToXMI1(QDomDocument & qDoc, QDomElement & qElement);
+    void saveToXMI1(QXmlStreamWriter& writer);
 
     virtual bool showPropertiesDialog(QWidget* parent = 0);
 
