@@ -4,7 +4,7 @@
  *   the Free Software Foundation; either version 2 of the License, or     *
  *   (at your option) any later version.                                   *
  *                                                                         *
- *   copyright (C) 2004-2020                                               *
+ *   copyright (C) 2004-2021                                               *
  *   Umbrello UML Modeller Authors <umbrello-devel@kde.org>                *
  ***************************************************************************/
 
@@ -24,6 +24,7 @@
 #include "listpopupmenu.h"
 #include "object_factory.h"
 #include "operation.h"
+#include "optionstate.h"
 #include "template.h"
 #include "uml.h"
 #include "umldoc.h"
@@ -32,6 +33,7 @@
 
 // qt includes
 #include <QPainter>
+#include <QXmlStreamWriter>
 
 DEBUG_REGISTER_DISABLED(ClassifierWidget)
 
@@ -39,13 +41,13 @@ const int ClassifierWidget::CIRCLE_SIZE = 30;
 const int ClassifierWidget::SOCKET_INCREMENT = 10;
 
 /**
- * Constructs a ClassifierWidget.
+ * Constructs a ClassifierWidget for a UMLClassifier.
  *
  * @param scene   The parent of this ClassifierWidget.
- * @param c       The UMLClassifier to represent.
+ * @param umlc    The UMLClassifier to represent.
  */
-ClassifierWidget::ClassifierWidget(UMLScene * scene, UMLClassifier *c)
-  : UMLWidget(scene, WidgetBase::wt_Class, c),
+ClassifierWidget::ClassifierWidget(UMLScene * scene, UMLClassifier *umlc)
+  : UMLWidget(scene, WidgetBase::wt_Class, umlc),
     m_pAssocWidget(nullptr),
     m_pInterfaceName(nullptr)
 {
@@ -72,19 +74,48 @@ ClassifierWidget::ClassifierWidget(UMLScene * scene, UMLClassifier *c)
         m_operationSignature = Uml::SignatureType::SigNoVis;
 
     setVisualPropertyCmd(ShowAttributes, ops.classState.showAtts);
-    setVisualPropertyCmd(ShowStereotype, ops.classState.showStereoType);
+
+    // Do not call setShowStereotype here, it is a virtual method
+    // and setup of the vtbl_ptr has not yet been finalized.
+    m_showStereotype = ops.classState.showStereoType;
+    if (m_showStereotype != Uml::ShowStereoType::None)
+        m_visualProperties |= ShowStereotype;
+
     setVisualPropertyCmd(DrawAsCircle, false);
 
     setShowAttSigs(ops.classState.showAttSig);
 
-    if (c && c->isInterface()) {
+    if (umlc && umlc->isInterface()) {
         setBaseType(WidgetBase::wt_Interface);
-        m_visualProperties = ShowOperations | ShowVisibility | ShowStereotype;
-        setShowStereotype(true);
+        m_visualProperties = ShowOperations | ShowVisibility;
+        setShowStereotype(Uml::ShowStereoType::Tags);
         updateSignatureTypes();
     }
+}
 
-    if (c && scene->isObjectDiagram()) {
+/**
+ * Constructs a ClassifierWidget for a UMLInstance.
+ *
+ * @param scene   The parent of this ClassifierWidget.
+ * @param umli    The UMLInstance to represent.
+ */
+ClassifierWidget::ClassifierWidget(UMLScene * scene, UMLInstance * umli)
+  : UMLWidget(scene, WidgetBase::wt_Instance, umli),
+    m_pAssocWidget(nullptr),
+    m_pInterfaceName(nullptr)
+{
+    DiagramProxyWidget::setShowLinkedDiagram(false);
+    const Settings::OptionState& ops = m_scene->optionState();
+    setVisualPropertyCmd(ShowVisibility, ops.classState.showVisibility);
+    setVisualPropertyCmd(ShowPublicOnly, ops.classState.showPublicOnly);
+    setVisualPropertyCmd(ShowPackage,    ops.classState.showPackage);
+    m_attributeSignature = Uml::SignatureType::ShowSig;
+
+    setVisualPropertyCmd(ShowAttributes, ops.classState.showAtts);
+
+    setShowAttSigs(ops.classState.showAttSig);
+
+    if (umli) {
         setBaseType(WidgetBase::wt_Instance);
         m_visualProperties =  ShowAttributes;
         updateSignatureTypes();
@@ -92,7 +123,7 @@ ClassifierWidget::ClassifierWidget(UMLScene * scene, UMLClassifier *c)
 }
 
 /**
- * Constructs a ClassifierWidget.
+ * Constructs a ClassifierWidget for a UMLPackage.
  *
  * @param scene   The parent of this ClassifierWidget.
  * @param o       The UMLPackage to represent.
@@ -121,7 +152,7 @@ ClassifierWidget::ClassifierWidget(UMLScene * scene, UMLPackage *o)
         m_operationSignature = Uml::SignatureType::SigNoVis;
 
     setVisualPropertyCmd(ShowAttributes, ops.classState.showAtts);
-    setVisualPropertyCmd(ShowStereotype, ops.classState.showStereoType);
+    setShowStereotype(ops.classState.showStereoType);
     setVisualPropertyCmd(DrawAsPackage, true);
 
     setShowAttSigs(ops.classState.showAttSig);
@@ -149,6 +180,18 @@ UMLClassifier *ClassifierWidget::classifier() const
     return m_umlObject->asUMLClassifier();
 }
 
+/**
+ * Reimplement method from UMLWidget.
+ */
+void ClassifierWidget::setShowStereotype(Uml::ShowStereoType::Enum flag)
+{
+    if (flag == Uml::ShowStereoType::None)
+        m_visualProperties &= ~ShowStereotype;
+    else
+        m_visualProperties |= ShowStereotype;
+    UMLWidget::setShowStereotype(flag);
+}
+
 /**
  * @return the visual properties
  */
@@ -186,11 +229,15 @@ bool ClassifierWidget::visualProperty(VisualProperty property) const
                 || m_attributeSignature == Uml::SignatureType::SigNoVis);
     }
 
-    else if(property == ShowOperationSignature) {
+    else if (property == ShowOperationSignature) {
         return (m_operationSignature == Uml::SignatureType::ShowSig
                 || m_operationSignature == Uml::SignatureType::SigNoVis);
     }
 
+    else if (property == ShowStereotype) {
+        return (m_showStereotype != Uml::ShowStereoType::None);
+    }
+
     return m_visualProperties.testFlag(property);
 }
 
@@ -249,15 +296,10 @@ void ClassifierWidget::setVisualPropertyCmd(VisualProperty property, bool enable
         //:TODO: updateTextItemGroups();
         updateSignatureTypes();
     }
-
+ 
     else if (property == ShowStereotype) {
-        // Now just update flag and use base method for actual work.
-        if (enable) {
-            m_visualProperties |= property;
-        } else {
-            m_visualProperties &= ~property;
-        }
-        setShowStereotype(enable);
+        setShowStereotype(enable ? Uml::ShowStereoType::Tags
+                                 : Uml::ShowStereoType::None);
     }
 
     else if (property == DrawAsCircle) {
@@ -452,11 +494,13 @@ int ClassifierWidget::displayedMembers(UMLObject::ObjectType ot) const
 {
     int count = 0;
     UMLClassifier *umlc = this->classifier();
+    if (!umlc && m_umlObject && m_umlObject->isUMLInstance() && ot == UMLObject::ot_Attribute)
+        umlc = m_umlObject->asUMLInstance()->classifier();
     if (!umlc)
         return count;
     UMLClassifierListItemList list = umlc->getFilteredList(ot);
     foreach (UMLClassifierListItem *m, list) {
-      if (!(visualProperty(ShowPublicOnly) && m->visibility() != Uml::Visibility::Public))
+        if (!(visualProperty(ShowPublicOnly) && m->visibility() != Uml::Visibility::Public))
             count++;
     }
     return count;
@@ -483,12 +527,11 @@ QSizeF ClassifierWidget::calculateSize(bool withExtensions /* = true */) const
         return calculateAsPackageSize();
     }
     UMLClassifier *umlc = this->classifier();
-    if (!umlc) {
-        uError() << "Internal error - classifier() returns NULL";
-        return UMLWidget::minimumSize();
-    }
-    if (umlc->isInterface() && visualProperty(DrawAsCircle)) {
-        return calculateAsCircleSize();
+    if (umlc) {
+        if (umlc->isInterface() && visualProperty(DrawAsCircle))
+            return calculateAsCircleSize();
+    } else if (m_umlObject && m_umlObject->isUMLInstance()) {
+       umlc = m_umlObject->asUMLInstance()->classifier();
     }
 
     const bool showNameOnly = !visualProperty(ShowAttributes) &&
@@ -500,13 +543,25 @@ QSizeF ClassifierWidget::calculateSize(bool withExtensions /* = true */) const
     // width is the width of the longest 'word'
     int width = 0, height = 0;
     // consider stereotype
-    if (visualProperty(ShowStereotype) && !m_umlObject->stereotype().isEmpty()) {
+    if (m_showStereotype != Uml::ShowStereoType::None &&
+                             m_umlObject && !m_umlObject->stereotype().isEmpty()) {
         height += fontHeight;
+        QString taggedValues;
+        if (m_showStereotype == Uml::ShowStereoType::Tags) {
+            taggedValues = tags();
+            if (!taggedValues.isEmpty())
+                height += fontHeight;
+        }
         // ... width
         const QFontMetrics &bfm = UMLWidget::getFontMetrics(UMLWidget::FT_BOLD);
         const int stereoWidth = bfm.size(0, m_umlObject->stereotype(true)).width();
         if (stereoWidth > width)
             width = stereoWidth;
+        if (!taggedValues.isEmpty()) {
+            const int tagsWidth = bfm.size(0, taggedValues).width();
+            if (tagsWidth > width)
+                width = tagsWidth;
+        }
     } else if (showNameOnly) {
         height += defaultMargin;
     }
@@ -516,25 +571,24 @@ QSizeF ClassifierWidget::calculateSize(bool withExtensions /* = true */) const
     // ... width
     QString name;
     UMLObject *o;
-    if (m_umlObject && m_umlObject->isUMLInstance() && m_umlObject->asUMLInstance()->classifier())
-        o = m_umlObject->asUMLInstance()->classifier();
+    if (m_umlObject && m_umlObject->isUMLInstance() && umlc)
+        o = umlc;
     else
         o = m_umlObject;
     if (!o)
         name = m_Text;
-    else
-    if (visualProperty(ShowPackage))
+    else if (visualProperty(ShowPackage))
         name = o->fullyQualifiedName();
     else
         name = o->name();
 
     QString displayedName;
-    if (m_umlObject->isUMLInstance())
+    if (m_umlObject && m_umlObject->isUMLInstance())
         displayedName = m_umlObject->name() + QLatin1String(" : ") + name;
     else
         displayedName = name;
 
-    const UMLWidget::FontType nft = (m_umlObject->isAbstract() ? FT_BOLD_ITALIC : FT_BOLD);
+    const UMLWidget::FontType nft = (m_umlObject && m_umlObject->isAbstract() ? FT_BOLD_ITALIC : FT_BOLD);
     const int nameWidth = UMLWidget::getFontMetrics(nft).size(0, displayedName).width();
     if (nameWidth > width)
         width = nameWidth;
@@ -561,8 +615,7 @@ QSizeF ClassifierWidget::calculateSize(bool withExtensions /* = true */) const
         if (numAtts > 0) {
             height += fontHeight * numAtts;
             // calculate width of the attributes
-            UMLClassifierListItemList list = umlc->getFilteredList(
-                        m_umlObject->isUMLInstance() ? UMLObject::ot_InstanceAttribute : UMLObject::ot_Attribute);
+            UMLClassifierListItemList list = umlc->getFilteredList(UMLObject::ot_Attribute);
             foreach (UMLClassifierListItem *a, list) {
                 if (visualProperty(ShowPublicOnly) && a->visibility() != Uml::Visibility::Public)
                     continue;
@@ -665,9 +718,6 @@ int ClassifierWidget::displayedAttributes() const
 {
     if (!visualProperty(ShowAttributes))
         return 0;
-    if(baseType() == WidgetBase::wt_Instance)
-        return displayedMembers(UMLObject::ot_InstanceAttribute);
-    else
     return displayedMembers(UMLObject::ot_Attribute);
 }
 
@@ -688,7 +738,7 @@ int ClassifierWidget::displayedOperations() const
 void ClassifierWidget::setClassAssociationWidget(AssociationWidget *assocwidget)
 {
     if (!classifier()) {
-        uError() << "Class association cannot be applied to package";
+        uError() << "Class association cannot be applied to non classifier";
         return;
     }
     m_pAssocWidget = assocwidget;
@@ -725,10 +775,14 @@ void ClassifierWidget::paint(QPainter *painter, const QStyleOptionGraphicsItem *
     }
     UMLClassifier *umlc = this->classifier();
     if (!umlc) {
-        uError() << "Internal error - classifier() returns NULL";
-        return;
+        if (m_umlObject && m_umlObject->isUMLInstance()) {
+            umlc = m_umlObject->asUMLInstance()->classifier();
+        } else {
+            uError() << "ClassifierWidget::paint internal error - classifier() returns NULL";
+            return;
+        }
     }
-    if (umlc->isInterface() && visualProperty(DrawAsCircle)) {
+    if (umlc && umlc->isInterface() && visualProperty(DrawAsCircle)) {
         drawAsCircle(painter, option);
         UMLWidget::paint(painter, option, widget);
         return;
@@ -754,7 +808,9 @@ void ClassifierWidget::paint(QPainter *painter, const QStyleOptionGraphicsItem *
     const int fontHeight = fm.lineSpacing();
 
     //If there are any templates then draw them
-    UMLTemplateList tlist = umlc->getTemplateList();
+    UMLTemplateList tlist;
+    if (umlc)
+        tlist = umlc->getTemplateList();
     if (tlist.count() > 0) {
         setPenFromSettings(painter);
         QPen pen = painter->pen();
@@ -769,7 +825,7 @@ void ClassifierWidget::paint(QPainter *painter, const QStyleOptionGraphicsItem *
         int y = defaultMargin;
         const int templateWidth = templatesBoxSize.width() - 2 * defaultMargin;
         foreach (UMLTemplate *t, tlist) {
-            QString text = t->toString(Uml::SignatureType::NoSig, visualProperty(ShowStereotype));
+            QString text = t->toString(Uml::SignatureType::NoSig, m_showStereotype != Uml::ShowStereoType::None);
             painter->drawText(x, y, templateWidth, fontHeight, Qt::AlignVCenter, text);
             y += fontHeight;
         }
@@ -787,33 +843,41 @@ void ClassifierWidget::paint(QPainter *painter, const QStyleOptionGraphicsItem *
                               !visualProperty(ShowDocumentation);
 
     int nameHeight = fontHeight;
-    if (visualProperty(ShowStereotype) && !m_umlObject->stereotype().isEmpty()) {
+    if (m_showStereotype != Uml::ShowStereoType::None && !m_umlObject->stereotype().isEmpty()) {
         painter->setFont(font);
         painter->drawText(textX, bodyOffsetY, textWidth, fontHeight, Qt::AlignCenter, m_umlObject->stereotype(true));
         bodyOffsetY += fontHeight;
+        if (m_showStereotype == Uml::ShowStereoType::Tags) {
+            QString taggedValues = tags();
+            if (!taggedValues.isEmpty()) {
+                painter->drawText(textX, bodyOffsetY, textWidth, fontHeight, Qt::AlignCenter, taggedValues);
+                bodyOffsetY += fontHeight;
+            }
+        }
     } else if (showNameOnly) {
         nameHeight = h;
     }
 
     // draw name
-    QString name;
-    UMLObject *o;
-    if (m_umlObject && m_umlObject->isUMLInstance() && m_umlObject->asUMLInstance()->classifier())
-        o = m_umlObject->asUMLInstance()->classifier();
-    else
-        o = m_umlObject;
-    if (!o)
-        name = m_Text;
-    else if (visualProperty(ShowPackage))
-        name = o->fullyQualifiedName();
-    else
-        name = o->name();
-
     QString displayedName;
-    if (m_umlObject->isUMLInstance())
-        displayedName = m_umlObject->name() + QLatin1String(" : ") + name;
-    else
-        displayedName = name;
+    if (!m_umlObject) {
+        displayedName = m_Text;
+    } else if (m_umlObject->isUMLInstance()) {
+        displayedName = m_umlObject->name() + QLatin1String(" : ");
+        if (umlc) {
+            if (visualProperty(ShowPackage))
+                displayedName.append(umlc->fullyQualifiedName());
+            else
+                displayedName.append(umlc->name());
+        }
+    } else if (visualProperty(ShowPackage)) {
+        displayedName = m_umlObject->fullyQualifiedName();
+    } else {
+        displayedName = m_umlObject->name();
+    }
+
+    if (baseType() == WidgetBase::wt_Object || baseType() == WidgetBase::wt_Instance)
+        font.setUnderline(true);
 
     font.setItalic(m_umlObject->isAbstract());
     painter->setFont(font);
@@ -821,6 +885,7 @@ void ClassifierWidget::paint(QPainter *painter, const QStyleOptionGraphicsItem *
     bodyOffsetY += fontHeight;
     font.setBold(false);
     font.setItalic(false);
+    font.setUnderline(false);
     painter->setFont(font);
 
 #ifdef ENABLE_WIDGET_SHOW_DOC
@@ -851,13 +916,8 @@ void ClassifierWidget::paint(QPainter *painter, const QStyleOptionGraphicsItem *
 
         const int numAtts = displayedAttributes();
         if (numAtts > 0) {
-            if (baseType() == WidgetBase::wt_Instance) {
-                drawMembers(painter, UMLObject::ot_InstanceAttribute, m_attributeSignature, textX,
-                            bodyOffsetY, textWidth, fontHeight);
-            } else {
-                drawMembers(painter, UMLObject::ot_Attribute, m_attributeSignature, textX,
-                            bodyOffsetY, textWidth, fontHeight);
-            }
+            drawMembers(painter, UMLObject::ot_Attribute, m_attributeSignature, textX,
+                        bodyOffsetY, textWidth, fontHeight);
             bodyOffsetY += fontHeight * numAtts;
         }
         else
@@ -1078,22 +1138,49 @@ QSize ClassifierWidget::calculateAsPackageSize() const
  * @param width      The text width.
  * @param height     The text height.
  */
-void ClassifierWidget::drawMembers(QPainter * painter, UMLObject::ObjectType ot, Uml::SignatureType::Enum sigType,
+void ClassifierWidget::drawMembers(QPainter * painter,
+                                   UMLObject::ObjectType ot,
+                                   Uml::SignatureType::Enum sigType,
                                    int x, int y, int width, int height)
 {
     UMLClassifier *umlc = classifier();
-    if (!umlc) {
+    const bool drawInstanceAttributes = (!umlc && m_umlObject && m_umlObject->isUMLInstance()
+                                                            && ot == UMLObject::ot_Attribute);
+    if (!umlc && !drawInstanceAttributes) {
         return;
     }
     QFont f = UMLWidget::font();
     f.setBold(false);
-    UMLClassifierListItemList list = umlc->getFilteredList(ot);
     painter->setClipping(true);
     painter->setClipRect(rect());
-    foreach (UMLClassifierListItem *obj, list) {
-          if (visualProperty(ShowPublicOnly) && obj->visibility() != Uml::Visibility::Public)
+    UMLObjectList ialist;
+    if (drawInstanceAttributes) {
+        UMLInstance *umlinst = m_umlObject->asUMLInstance();
+        umlc = umlinst->classifier();
+        ialist = umlinst->subordinates();
+    }
+    UMLClassifierListItemList list = umlc->getFilteredList(ot);
+    for (int i = 0; i < list.count(); i++) {
+        UMLClassifierListItem *obj = list.at(i);
+        uIgnoreZeroPointer(obj);
+        if (visualProperty(ShowPublicOnly) && obj->visibility() != Uml::Visibility::Public)
             continue;
-        QString text = obj->toString(sigType, visualProperty(ShowStereotype));
+        QString text;
+        if (drawInstanceAttributes) {
+            UMLInstanceAttribute *iatt = ialist.at(i)->asUMLInstanceAttribute();
+            if (!iatt) {
+                uDebug() << "ClassifierWidget::drawMembers(" << obj->name()
+                         << ") : skipping non InstanceAttribute subordinate";
+                continue;
+            }
+            /* CHECK: Do we want visibility indication on instance attributes?
+            if (sigType == Uml::SignatureType::ShowSig || sigType == Uml::SignatureType::NoSig)
+                text = Uml::Visibility::toString(iatt->visibility(), true) + QLatin1Char(' ');
+             */
+            text.append(iatt->toString());
+        } else {
+            text = obj->toString(sigType, visualProperty(ShowStereotype));
+        }
         f.setItalic(obj->isAbstract());
         f.setUnderline(obj->isStatic());
         painter->setFont(f);
@@ -1216,7 +1303,7 @@ void ClassifierWidget::changeToClass()
     setVisualPropertyCmd(DrawAsCircle, false);
     const Settings::OptionState& ops = m_scene->optionState();
     setVisualProperty(ShowAttributes, ops.classState.showAtts);
-    setVisualProperty(ShowStereotype, ops.classState.showStereoType);
+    setShowStereotype(ops.classState.showStereoType);
 
     updateGeometry();
     update();
@@ -1233,7 +1320,7 @@ void ClassifierWidget::changeToInterface()
     m_umlObject->setBaseType(UMLObject::ot_Interface);
 
     setVisualProperty(ShowAttributes, false);
-    setVisualProperty(ShowStereotype, true);
+    setShowStereotype(Settings::optionState().classState.showStereoType);
 
     updateGeometry();
     update();
@@ -1249,7 +1336,7 @@ void ClassifierWidget::changeToPackage()
     m_umlObject->setBaseType(UMLObject::ot_Package);
 
     setVisualProperty(ShowAttributes, false);
-    setVisualProperty(ShowStereotype, true);
+    setShowStereotype(Uml::ShowStereoType::Name);
 
     updateGeometry();
     update();
@@ -1286,7 +1373,7 @@ bool ClassifierWidget::loadFromXMI1(QDomElement & qElement)
         setVisualPropertyCmd(ShowPackage,    (bool)showpackage.toInt());
         setVisualPropertyCmd(ShowVisibility, (bool)showscope.toInt());
         setVisualPropertyCmd(DrawAsCircle,   (bool)drawascircle.toInt());
-        setVisualPropertyCmd(ShowStereotype, (bool)showstereotype.toInt());
+        setShowStereotype((Uml::ShowStereoType::Enum)showstereotype.toInt());
         m_attributeSignature = Uml::SignatureType::fromInt(showattsigs.toInt());
         m_operationSignature = Uml::SignatureType::fromInt(showopsigs.toInt());
     }
@@ -1330,43 +1417,45 @@ bool ClassifierWidget::loadFromXMI1(QDomElement & qElement)
 /**
  * Creates the "classwidget" or "interfacewidget" XML element.
  */
-void ClassifierWidget::saveToXMI1(QDomDocument & qDoc, QDomElement & qElement)
+void ClassifierWidget::saveToXMI1(QXmlStreamWriter& writer)
 {
-    QDomElement conceptElement;
     bool saveShowAttributes = true;
     UMLClassifier *umlc = classifier();
-    if (umlObject() && umlObject()->baseType() == UMLObject::ot_Package) {
-        conceptElement = qDoc.createElement(QLatin1String("packagewidget"));
+    QString tag;
+    if (umlObject()->baseType() == UMLObject::ot_Package) {
+        tag = QLatin1String("packagewidget");
         saveShowAttributes = false;
-    } else if(umlObject()->baseType() == UMLObject::ot_Instance) {
-        conceptElement = qDoc.createElement(QLatin1String("instancewidget"));
+    } else if (umlObject()->baseType() == UMLObject::ot_Instance) {
+        tag = QLatin1String("instancewidget");
         saveShowAttributes = false;
     } else if (umlc && umlc->isInterface()) {
-        conceptElement = qDoc.createElement(QLatin1String("interfacewidget"));
+        tag = QLatin1String("interfacewidget");
     } else {
-        conceptElement = qDoc.createElement(QLatin1String("classwidget"));
+        tag = QLatin1String("classwidget");
     }
-    UMLWidget::saveToXMI1(qDoc, conceptElement);
+    writer.writeStartElement(tag);
+
+    UMLWidget::saveToXMI1(writer);
     if (saveShowAttributes) {
-        conceptElement.setAttribute(QLatin1String("showoperations"), visualProperty(ShowOperations));
-        conceptElement.setAttribute(QLatin1String("showpubliconly"), visualProperty(ShowPublicOnly));
-        conceptElement.setAttribute(QLatin1String("showopsigs"),     m_operationSignature);
-        conceptElement.setAttribute(QLatin1String("showpackage"),    visualProperty(ShowPackage));
-        conceptElement.setAttribute(QLatin1String("showscope"),      visualProperty(ShowVisibility));
-        conceptElement.setAttribute(QLatin1String("showattributes"), visualProperty(ShowAttributes));
-        conceptElement.setAttribute(QLatin1String("showattsigs"),    m_attributeSignature);
-        conceptElement.setAttribute(QLatin1String("showstereotype"), visualProperty(ShowStereotype));
+        writer.writeAttribute(QLatin1String("showoperations"), QString::number(visualProperty(ShowOperations)));
+        writer.writeAttribute(QLatin1String("showpubliconly"), QString::number(visualProperty(ShowPublicOnly)));
+        writer.writeAttribute(QLatin1String("showopsigs"),     QString::number(m_operationSignature));
+        writer.writeAttribute(QLatin1String("showpackage"),    QString::number(visualProperty(ShowPackage)));
+        writer.writeAttribute(QLatin1String("showscope"),      QString::number(visualProperty(ShowVisibility)));
+        writer.writeAttribute(QLatin1String("showattributes"), QString::number(visualProperty(ShowAttributes)));
+        writer.writeAttribute(QLatin1String("showattsigs"),    QString::number(m_attributeSignature));
+        writer.writeAttribute(QLatin1String("showstereotype"), QString::number(m_showStereotype));
     }
 #ifdef ENABLE_WIDGET_SHOW_DOC
-    conceptElement.setAttribute(QLatin1String("showdocumentation"),visualProperty(ShowDocumentation));
+    writer.writeAttribute(QLatin1String("showdocumentation"), QString::number(visualProperty(ShowDocumentation)));
 #endif
     if (umlc && (umlc->isInterface() || umlc->isAbstract())) {
-        conceptElement.setAttribute(QLatin1String("drawascircle"), visualProperty(DrawAsCircle));
+        writer.writeAttribute(QLatin1String("drawascircle"), QString::number(visualProperty(DrawAsCircle)));
         if (visualProperty(DrawAsCircle) && m_pInterfaceName) {
-            m_pInterfaceName->saveToXMI1(qDoc, conceptElement);
+            m_pInterfaceName->saveToXMI1(writer);
         }
     }
-    qElement.appendChild(conceptElement);
+    writer.writeEndElement();
 }
 
 /**
@@ -1378,11 +1467,11 @@ void ClassifierWidget::saveToXMI1(QDomDocument & qDoc, QDomElement & qElement)
 void ClassifierWidget::slotMenuSelection(QAction* action)
 {
     ListPopupMenu::MenuType sel = ListPopupMenu::typeFromAction(action);
+    uDebug() << "ClassifierWidget::slotMenuSelection sel = " << sel;
     switch (sel) {
     case ListPopupMenu::mt_Attribute:
     case ListPopupMenu::mt_Operation:
     case ListPopupMenu::mt_Template:
-    case ListPopupMenu::mt_InstanceAttribute:
         {
             UMLObject::ObjectType ot = ListPopupMenu::convert_MT_OT(sel);
             UMLClassifier *umlc = classifier();
