@@ -4,7 +4,7 @@
  *   the Free Software Foundation; either version 2 of the License, or     *
  *   (at your option) any later version.                                   *
  *                                                                         *
- *   copyright (C) 2003-2014                                               *
+ *   copyright (C) 2003-2020                                               *
  *   Umbrello UML Modeller Authors <umbrello-devel@kde.org>                *
  ***************************************************************************/
 
@@ -25,6 +25,11 @@
 #include "umlscene.h"
 #include "umlview.h"
 
+// qt includes
+#include <QXmlStreamWriter>
+
+DEBUG_REGISTER_DISABLED(EnumWidget)
+
 /**
  * Constructs an instance of EnumWidget.
  *
@@ -170,13 +175,13 @@ bool EnumWidget::loadFromXMI1(QDomElement & qElement)
 /**
  * Saves to the "enumwidget" XMI element.
  */
-void EnumWidget::saveToXMI1(QDomDocument& qDoc, QDomElement& qElement)
+void EnumWidget::saveToXMI1(QXmlStreamWriter& writer)
 {
-    QDomElement conceptElement = qDoc.createElement(QLatin1String("enumwidget"));
-    UMLWidget::saveToXMI1(qDoc, conceptElement);
+    writer.writeStartElement(QLatin1String("enumwidget"));
+    UMLWidget::saveToXMI1(writer);
 
-    conceptElement.setAttribute(QLatin1String("showpackage"), m_showPackage);
-    qElement.appendChild(conceptElement);
+    writer.writeAttribute(QLatin1String("showpackage"), QString::number(m_showPackage));
+    writer.writeEndElement();
 }
 
 /**
