@@ -4,7 +4,7 @@
  *   the Free Software Foundation; either version 2 of the License, or     *
  *   (at your option) any later version.                                   *
  *                                                                         *
- *   copyright (C) 2014-2020                                               *
+ *   copyright (C) 2014-2021                                               *
  *   Umbrello UML Modeller Authors <umbrello-devel@kde.org>                *
  ***************************************************************************/
 
@@ -26,6 +26,7 @@
 // qt includes
 #include <QPainter>
 #include <QToolTip>
+#include <QXmlStreamWriter>
 
 // sys includes
 #include <cmath>
@@ -93,6 +94,21 @@ void PinPortBase::setInitialPosition(const QPointF &scenePos)
     m_childPlacement->setInitialPosition(scenePos);
 }
 
+qreal PinPortBase::getX() const
+{
+    return parentItem()->x() + UMLWidget::getX();
+}
+
+qreal PinPortBase::getY() const
+{
+    return parentItem()->y() + UMLWidget::getY();
+}
+
+QPointF PinPortBase::getPos() const
+{
+    return parentItem()->pos() + UMLWidget::getPos();
+}
+
 /**
  * Overrides method from UMLWidget in order to set a tooltip.
  * The tooltip is set to the name().
@@ -266,17 +282,18 @@ UMLWidget* PinPortBase::widgetWithID(Uml::ID::Type id)
 /**
  * Saves the widget to the "pinwidget" or "portwidget" XMI element.
  */
-void PinPortBase::saveToXMI1(QDomDocument& qDoc, QDomElement& qElement)
+void PinPortBase::saveToXMI1(QXmlStreamWriter& writer)
 {
-    QDomElement element = qDoc.createElement(baseType() == wt_Pin ? QLatin1String("pinwidget")
-                                                                  : QLatin1String("portwidget"));
+    QString tag = (baseType() == wt_Pin ? QLatin1String("pinwidget")
+                                        : QLatin1String("portwidget"));
+    writer.writeStartElement(tag);
     Q_ASSERT(ownerWidget() != 0);
-    element.setAttribute(QLatin1String("widgetaid"), Uml::ID::toString(ownerWidget()->id()));
-    UMLWidget::saveToXMI1(qDoc, element);
+    writer.writeAttribute(QLatin1String("widgetaid"), Uml::ID::toString(ownerWidget()->id()));
+    UMLWidget::saveToXMI1(writer);
     if (m_pName && !m_pName->text().isEmpty()) {
-        m_pName->saveToXMI1(qDoc, element);
+        m_pName->saveToXMI1(writer);
     }
-    qElement.appendChild(element);
+    writer.writeEndElement();
 }
 
 /**
