@@ -4,7 +4,7 @@
  *   the Free Software Foundation; either version 2 of the License, or     *
  *   (at your option) any later version.                                   *
  *                                                                         *
- *   copyright (C) 2004-2014                                               *
+ *   copyright (C) 2004-2020                                               *
  *   Umbrello UML Modeller Authors <umbrello-devel@kde.org>                *
  ***************************************************************************/
 
@@ -24,10 +24,11 @@
 
 class QGraphicsItem;
 class QGraphicsRectItem;
+class QXmlStreamWriter;
 
 /**
  * General purpose widget utilities.
- * Bugs and comments to umbrello-devel@kde.org or http://bugs.kde.org
+ * Bugs and comments to umbrello-devel@kde.org or https://bugs.kde.org
  */
 namespace Widget_Utils
 {
@@ -49,14 +50,13 @@ namespace Widget_Utils
     QPointF stringToPoint(const QString& str);
 
     bool loadPixmapFromXMI(QDomElement &qElement, QPixmap &pixmap);
-    void savePixmapToXMI(QDomDocument &qDoc, QDomElement &qElement, const QPixmap& pixmap);
+    void savePixmapToXMI(QXmlStreamWriter& stream, const QPixmap& pixmap);
 
     bool loadGradientFromXMI(QDomElement &qElement, QGradient *&gradient);
-    void saveGradientToXMI(QDomDocument &qDoc, QDomElement &qElement, const QGradient *gradient);
+    void saveGradientToXMI(QXmlStreamWriter& stream, const QGradient *gradient);
 
     bool loadBrushFromXMI(QDomElement &qElement, QBrush &brush);
-    void saveBrushToXMI(QDomDocument &qDoc, QDomElement &qElement,
-                        const QBrush& brush);
+    void saveBrushToXMI(QXmlStreamWriter& stream, const QBrush& brush);
 
     bool hasSmallerX(const UMLWidget* widget1, const UMLWidget* widget2);
     bool hasSmallerY(const UMLWidget* widget1, const UMLWidget* widget2);
